package be.jeffcheasey88.peeratcode.parser.java.operations;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import be.jeffcheasey88.peeratcode.parser.java.CleanerPool;
import be.jeffcheasey88.peeratcode.parser.java.JavaElement;
import be.jeffcheasey88.peeratcode.parser.java.CleanerPool.Cleaner;

public class ConditionalOperation extends OperationContainer{
	
	private Pattern pattern;
	private String condition;
	
	public ConditionalOperation(Pattern pattern){
		this.pattern = pattern;
	}

	@Override
	public int parse(String content, CleanerPool cleaner) throws Exception{
		CleanerPool generic = new CleanerPool(
				new Cleaner("GENERIC_PARENTHESIS",'(',')'),
				new Cleaner("GENERIC_FUNCTION", '{','}'));
		content = generic.clean(content);
		
		Matcher matcher = this.pattern.matcher(content);
		matcher.matches();
		
		this.condition = generic.unzip(matcher.group(2), (s,p) -> s);
		
		int index = generic.unzip(matcher.group(1), (s,p) -> s).length();
		content = generic.unzip(content, (s,p) -> s).substring(index);
		int bodysize;
		if(content.startsWith("{")){
			bodysize = content.indexOf('}')+1;
			content = content.substring(1, bodysize-1);
			parse(content, cleaner, generic);
		}else{
			bodysize = parseOne(content, cleaner, generic);
		}
		
		return index+bodysize;
	}

	@Override
	public void show(int tab) {
		String start = "";
		for(int i = 0; i < tab; i++) start+="\t";
		System.out.println(start+"Condition??"+condition+"{");
		for(JavaElement child : getChilds()) child.show(tab+1);
		System.out.println(start+"}");
	}

	public static class IfOperation extends ConditionalOperation{

		private static Pattern PATTERN = Pattern.compile("^(\\s*if\\s*(\\^GENERIC_PARENTHESIS\\d+)).*$");
		
		public IfOperation(){
			super(PATTERN);
		}

		@Override
		public void show(int tab) {
			String start = "";
			for(int i = 0; i < tab; i++) start+="\t";
			System.out.println(start+"if"+super.condition+"{");
			for(JavaElement child : getChilds()) child.show(tab+1);
			System.out.println(start+"}");
		}
	}
	
	public static class ForOperation extends ConditionalOperation{

		private static Pattern PATTERN = Pattern.compile("^(\\s*for\\s*(\\^GENERIC_PARENTHESIS\\d+)).*$");
		
		public ForOperation(){
			super(PATTERN);
		}

		@Override
		public void show(int tab) {
			String start = "";
			for(int i = 0; i < tab; i++) start+="\t";
			System.out.println(start+"for"+super.condition+"{");
			for(JavaElement child : getChilds()) child.show(tab+1);
			System.out.println(start+"}");
		}
	}
	
	public static class WhileOperation extends ConditionalOperation{

		private static Pattern PATTERN = Pattern.compile("^(\\s*while\\s*(\\^GENERIC_PARENTHESIS\\d+)).*$");
		
		public WhileOperation(){
			super(PATTERN);
		}

		@Override
		public void show(int tab) {
			String start = "";
			for(int i = 0; i < tab; i++) start+="\t";
			System.out.println(start+"while"+super.condition+"{");
			for(JavaElement child : getChilds()) child.show(tab+1);
			System.out.println(start+"}");
		}
	}
}
